@@ -17,7 +17,7 @@
 """Python bindings for Subversion."""
 
 __author__ = "Jelmer Vernooij <jelmer@samba.org>"
-__version__ = (0, 8, 7)
+__version__ = (0, 8, 9)
 
 NODE_DIR = 2
 NODE_FILE = 1
@@ -39,6 +39,7 @@ ERR_REPOS_DISABLED_FEATURE = 165006
 ERR_STREAM_MALFORMED_DATA = 140001
 ERR_RA_ILLEGAL_URL = 170000
 ERR_RA_LOCAL_REPOS_OPEN_FAILED = 180001
+ERR_BAD_FILENAME = 125001
 ERR_BAD_URL = 125002
 ERR_BAD_DATE = 125003
 ERR_RA_DAV_REQUEST_FAILED = 175002
@@ -74,6 +75,7 @@ ERR_RA_SERF_SSL_CERT_UNTRUSTED = 230001
 ERR_ENTRY_NOT_FOUND = 150000
 ERR_BAD_PROPERTY_VALUE = 125005
 ERR_FS_ROOT_DIR = 160021
+ERR_WC_NODE_KIND_CHANGE = 155018
 
 ERR_APR_OS_START_EAIERR = 670000
 ERR_APR_OS_ERRSPACE_SIZE = 50000
