@@ -396,6 +396,8 @@ static svn_error_t *py_file_rev_handler(void *baton, const char *path, svn_revnu
 	if (delta_baton != NULL && delta_handler != NULL) {
 		*delta_baton = (void *)ret;
 		*delta_handler = py_txdelta_window_handler;
+	} else {
+		Py_DECREF(ret);
 	}
 	PyGILState_Release(state);
 	return NULL;
@@ -416,6 +418,8 @@ static svn_error_t *py_ra_file_rev_handler(void *baton, const char *path, svn_re
 	if (delta_baton != NULL && delta_handler != NULL) {
 		*delta_baton = (void *)ret;
 		*delta_handler = py_txdelta_window_handler;
+	} else {
+		Py_DECREF(ret);
 	}
 	PyGILState_Release(state);
 	return NULL;
@@ -527,8 +531,8 @@ static svn_error_t *py_open_tmp_file(apr_file_t **fp, void *callback,
 		Py_DECREF(ret);
 	} else if (PyFile_Check(ret)) {
 		*fp = apr_file_from_object(ret, pool);
+		Py_DECREF(ret);
 		if (!*fp) {
-			Py_DECREF(ret);
 			PyGILState_Release(state);
 			return py_svn_error();
 		}
@@ -1265,6 +1269,7 @@ static PyObject *get_commit_editor(PyObject *self, PyObject *args, PyObject *kwa
 	if (hash_revprops == NULL) {
 		apr_pool_destroy(pool);
 		ra->busy = false;
+		Py_DECREF(commit_callback);
 		return NULL;
 	}
 	Py_BEGIN_ALLOW_THREADS
@@ -1277,6 +1282,7 @@ static PyObject *get_commit_editor(PyObject *self, PyObject *args, PyObject *kwa
 	if (PyDict_Size(revprops) != 1) {
 		PyErr_SetString(PyExc_ValueError, "Only svn:log can be set with Subversion 1.4");
 		apr_pool_destroy(pool);
+		Py_DECREF(commit_callback);
 		ra->busy = false;
 		return NULL;
 	}
@@ -1285,6 +1291,7 @@ static PyObject *get_commit_editor(PyObject *self, PyObject *args, PyObject *kwa
 	if (py_log_msg == NULL) {
 		PyErr_SetString(PyExc_ValueError, "Only svn:log can be set with Subversion 1.4.");
 		apr_pool_destroy(pool);
+		Py_DECREF(commit_callback);
 		ra->busy = false;
 		return NULL;
 	}
@@ -1292,6 +1299,7 @@ static PyObject *get_commit_editor(PyObject *self, PyObject *args, PyObject *kwa
 	if (!PyString_Check(py_log_msg)) {
 		PyErr_SetString(PyExc_ValueError, "svn:log property should be set to string.");
 		apr_pool_destroy(pool);
+		Py_DECREF(commit_callback);
 		ra->busy = false;
 		return NULL;
 	}
@@ -1307,6 +1315,7 @@ static PyObject *get_commit_editor(PyObject *self, PyObject *args, PyObject *kwa
 	if (err != NULL) {
 		handle_svn_error(err);
 		svn_error_clear(err);
+		Py_DECREF(commit_callback);
 		apr_pool_destroy(pool);
 		ra->busy = false;
 		return NULL;
@@ -1387,16 +1396,34 @@ static PyObject *ra_get_dir(PyObject *self, PyObject *args, PyObject *kwargs)
 		Py_INCREF(py_dirents);
 	} else {
 		py_dirents = PyDict_New();
+		if (py_dirents == NULL) {
+			apr_pool_destroy(temp_pool);
+			return NULL;
+		}
 		idx = apr_hash_first(temp_pool, dirents);
 		while (idx != NULL) {
-			PyObject *item;
+			PyObject *item, *pykey;
 			apr_hash_this(idx, (const void **)&key, &klen, (void **)&dirent);
 			item = py_dirent(dirent, dirent_fields);
 			if (item == NULL) {
+				apr_pool_destroy(temp_pool);
 				Py_DECREF(py_dirents);
 				return NULL;
 			}
-			PyDict_SetItemString(py_dirents, key, item);
+			if (key == NULL) {
+				pykey = Py_None;
+				Py_INCREF(pykey);
+			} else {
+				pykey = PyString_FromString((char *)key);
+			}
+			if (PyDict_SetItem(py_dirents, pykey, item) != 0) {
+				Py_DECREF(py_dirents);
+				Py_DECREF(item);
+				Py_DECREF(pykey);
+				apr_pool_destroy(temp_pool);
+				return NULL;
+			}
+			Py_DECREF(pykey);
 			Py_DECREF(item);
 			idx = apr_hash_next(idx);
 		}
@@ -1404,6 +1431,7 @@ static PyObject *ra_get_dir(PyObject *self, PyObject *args, PyObject *kwargs)
 
 	py_props = prop_hash_to_dict(props);
 	if (py_props == NULL) {
+		Py_DECREF(py_dirents);
 		apr_pool_destroy(temp_pool);
 		return NULL;
 	}
@@ -1656,16 +1684,26 @@ static PyObject *ra_get_locks(PyObject *self, PyObject *args)
 	RUN_RA_WITH_POOL(temp_pool, ra, svn_ra_get_locks(ra->ra, &hash_locks, path, temp_pool));
 
 	ret = PyDict_New();
+	if (ret == NULL) {
+		apr_pool_destroy(temp_pool);
+		return NULL;
+	}
 	for (idx = apr_hash_first(temp_pool, hash_locks); idx != NULL;
 		 idx = apr_hash_next(idx)) {
 		PyObject *pyval;
 		apr_hash_this(idx, (const void **)&key, &klen, (void **)&lock);
 		pyval = pyify_lock(lock);
 		if (pyval == NULL) {
+			Py_DECREF(ret);
 			apr_pool_destroy(temp_pool);
 			return NULL;
 		}
-		PyDict_SetItemString(ret, key, pyval);
+		if (PyDict_SetItemString(ret, key, pyval) != 0) {
+			apr_pool_destroy(temp_pool);
+			Py_DECREF(pyval);
+			Py_DECREF(ret);
+			return NULL;
+		}
 		Py_DECREF(pyval);
 	}
 
@@ -1706,15 +1744,18 @@ static PyObject *ra_get_locations(PyObject *self, PyObject *args)
 					temp_pool));
 	ret = PyDict_New();
 	if (ret == NULL) {
-		PyErr_NoMemory();
 		apr_pool_destroy(temp_pool);
 		return NULL;
 	}
 
-	for (idx = apr_hash_first(temp_pool, hash_locations); idx != NULL; 
+	for (idx = apr_hash_first(temp_pool, hash_locations); idx != NULL;
 		idx = apr_hash_next(idx)) {
 		apr_hash_this(idx, (const void **)&key, &klen, (void **)&val);
-		PyDict_SetItem(ret, PyInt_FromLong(*key), PyString_FromString(val));
+		if (PyDict_SetItem(ret, PyInt_FromLong(*key), PyString_FromString(val)) != 0) {
+			Py_DECREF(ret);
+			apr_pool_destroy(temp_pool);
+			return NULL;
+		}
 	}
 	apr_pool_destroy(temp_pool);
 	return ret;
@@ -1732,32 +1773,53 @@ static PyObject *merge_rangelist_to_list(apr_array_header_t *rangelist)
 	int i;
 
 	ret = PyList_New(rangelist->nelts);
+	if (ret == NULL)
+		return NULL;
 
 	for (i = 0; i < rangelist->nelts; i++) {
-		PyObject *pyval = range_to_tuple(APR_ARRAY_IDX(rangelist, i, svn_merge_range_t *));
-		if (pyval == NULL)
+		PyObject *pyval;
+		pyval = range_to_tuple(APR_ARRAY_IDX(rangelist, i, svn_merge_range_t *));
+		if (pyval == NULL) {
+			Py_DECREF(ret);
 			return NULL;
-		PyList_SetItem(ret, i, pyval);
+		}
+		if (PyList_SetItem(ret, i, pyval) != 0) {
+			Py_DECREF(ret);
+			Py_DECREF(pyval);
+			return NULL;
+		}
 	}
+
 	return ret;
 }
 
 static PyObject *mergeinfo_to_dict(svn_mergeinfo_t mergeinfo, apr_pool_t *temp_pool)
 {
-	PyObject *ret = PyDict_New();
+	PyObject *ret;
 	char *key;
 	apr_ssize_t klen;
 	apr_hash_index_t *idx;
 	apr_array_header_t *range;
 
+	ret = PyDict_New();
+	if (ret == NULL) {
+		return NULL;
+	}
+
 	for (idx = apr_hash_first(temp_pool, mergeinfo); idx != NULL; 
 		idx = apr_hash_next(idx)) {
 		PyObject *pyval;
 		apr_hash_this(idx, (const void **)&key, &klen, (void **)&range);
 		pyval = merge_rangelist_to_list(range);
-		if (pyval == NULL)
+		if (pyval == NULL) {
+			Py_DECREF(ret);
 			return NULL;
-		PyDict_SetItemString(ret, key, pyval);
+		}
+		if (PyDict_SetItemString(ret, key, pyval) != 0) {
+			Py_DECREF(ret);
+			Py_DECREF(pyval);
+			return NULL;
+		}
 		Py_DECREF(pyval);
 	}
 
@@ -1800,6 +1862,10 @@ static PyObject *ra_mergeinfo(PyObject *self, PyObject *args)
                      temp_pool));
 
 	ret = PyDict_New();
+	if (ret == NULL) {
+		apr_pool_destroy(temp_pool);
+		return NULL;
+	}
 
 	if (catalog != NULL) {
 		for (idx = apr_hash_first(temp_pool, catalog); idx != NULL; 
@@ -1809,9 +1875,16 @@ static PyObject *ra_mergeinfo(PyObject *self, PyObject *args)
 			pyval = mergeinfo_to_dict(val, temp_pool);
 			if (pyval == NULL) {
 				apr_pool_destroy(temp_pool);
+				Py_DECREF(ret);
 				return NULL;
 			}
-			PyDict_SetItemString(ret, key, pyval);
+			if (PyDict_SetItemString(ret, key, pyval) != 0) {
+				apr_pool_destroy(temp_pool);
+				Py_DECREF(pyval);
+				Py_DECREF(ret);
+				return NULL;
+			}
+
 			Py_DECREF(pyval);
 		}
 	}
@@ -2834,8 +2907,10 @@ static PyObject *get_username_provider(PyObject *self)
 	if (auth == NULL)
 		return NULL;
 	auth->pool = Pool(NULL);
-	if (auth->pool == NULL)
+	if (auth->pool == NULL) {
+		PyObject_Del(auth);
 		return NULL;
+	}
 	svn_auth_get_username_provider(&auth->provider, auth->pool);
 	return (PyObject *)auth;
 }
@@ -2859,6 +2934,7 @@ static svn_error_t *py_cb_get_simple_provider_prompt(svn_boolean_t *may_save_pla
 			return py_svn_error();
 		}
 		*may_save_plaintext = PyObject_IsTrue(ret);
+		Py_DECREF(ret);
 		PyGILState_Release(state);
 	}
 
@@ -3027,9 +3103,14 @@ static PyObject *get_platform_specific_client_providers(PyObject *self)
 	const char *provider_types[] = {
 		"simple", "ssl_client_cert_pw", "ssl_server_trust", NULL,
 	};
-	PyObject *pylist = PyList_New(0);
+	PyObject *pylist;
 	int i, j;
 
+	pylist = PyList_New(0);
+	if (pylist == NULL) {
+		return NULL;
+	}
+
 	for (i = 0; provider_names[i] != NULL; i++) {
 		for (j = 0; provider_types[j] != NULL; j++) {
 			svn_auth_provider_object_t *c_provider = NULL;
@@ -3067,6 +3148,11 @@ static PyObject *get_platform_specific_client_providers(PyObject *self)
 	PyObject *pylist = PyList_New(0);
 	PyObject *provider = NULL;
 
+	if (pylist == NULL) {
+		Py_DECREF(pylist);
+		return NULL;
+	}
+
 #if defined(WIN32)
 	provider = get_windows_simple_provider(self);
 	if (provider == NULL)
