@@ -15,8 +15,10 @@
 
 """Subversion core library tests."""
 
-from unittest import TestCase
+import os
+import time
 from subvertpy import properties
+from subvertpy.tests import TestCase
 
 class TestProperties(TestCase):
 
@@ -26,6 +28,21 @@ class TestProperties(TestCase):
     def test_time_from_cstring(self):
         self.assertEquals(1225704780716938L, properties.time_from_cstring("2008-11-03T09:33:00.716938Z"))
 
+    def test_time_from_cstring_independent_from_dst(self):
+        old_tz = os.environ.get('TZ', None)
+        try:
+            # First specify a fixed timezone with known DST (late March to late October)
+            os.environ['TZ'] = 'Europe/London'
+            time.tzset()
+            # Now test a time within that DST
+            self.assertEquals(1275295762430000L, properties.time_from_cstring("2010-05-31T08:49:22.430000Z"))
+        finally:
+            if old_tz is None:
+                del os.environ['TZ']
+            else:
+                os.environ['TZ'] = old_tz
+            time.tzset()
+
     def test_time_to_cstring(self):
         self.assertEquals("2008-11-03T09:33:00.716938Z", properties.time_to_cstring(1225704780716938L))
 
