@@ -20,6 +20,8 @@
 #ifndef _SUBVERTPY_UTIL_H_
 #define _SUBVERTPY_UTIL_H_
 
+#include <svn_version.h>
+
 #if SVN_VER_MAJOR != 1
 #error "only svn 1.x is supported"
 #endif
@@ -41,8 +43,9 @@ typedef int Py_ssize_t;
 #pragma GCC visibility push(hidden)
 #endif
 
+svn_error_t *py_cancel_check(void *cancel_baton);
 __attribute__((warn_unused_result)) apr_pool_t *Pool(apr_pool_t *parent);
-__attribute__((warn_unused_result)) bool check_error(svn_error_t *error);
+void handle_svn_error(svn_error_t *error);
 bool string_list_to_apr_array(apr_pool_t *pool, PyObject *l, apr_array_header_t **);
 bool path_list_to_apr_array(apr_pool_t *pool, PyObject *l, apr_array_header_t **);
 PyObject *prop_hash_to_dict(apr_hash_t *props);
@@ -61,7 +64,9 @@ PyTypeObject *PyErr_GetSubversionExceptionTypeObject(void);
 	_save = PyEval_SaveThread(); \
 	err = (cmd); \
 	PyEval_RestoreThread(_save); \
-	if (!check_error(err)) { \
+	if (err != NULL) { \
+		handle_svn_error(err); \
+		svn_error_clear(err); \
 		return NULL; \
 	} \
 }
@@ -72,7 +77,9 @@ PyTypeObject *PyErr_GetSubversionExceptionTypeObject(void);
 	_save = PyEval_SaveThread(); \
 	err = (cmd); \
 	PyEval_RestoreThread(_save); \
-	if (!check_error(err)) { \
+	if (err != NULL) { \
+		handle_svn_error(err); \
+		svn_error_clear(err); \
 		apr_pool_destroy(pool); \
 		return NULL; \
 	} \
@@ -82,7 +89,6 @@ PyObject *wrap_lock(svn_lock_t *lock);
 apr_array_header_t *revnum_list_to_apr_array(apr_pool_t *pool, PyObject *l);
 svn_stream_t *new_py_stream(apr_pool_t *pool, PyObject *py);
 PyObject *PyErr_NewSubversionException(svn_error_t *error);
-svn_error_t *py_cancel_func(void *cancel_baton);
 apr_hash_t *config_hash_from_object(PyObject *config, apr_pool_t *pool);
 void PyErr_SetAprStatus(apr_status_t status);
 PyObject *py_dirent(const svn_dirent_t *dirent, int dirent_fields);
